@@ -2,18 +2,18 @@
  *	This file is part of devilspie2
  *	Copyright (C) 2005 Ross Burton, 2011-2012 Andreas Rönnquist
  *
- *	devilspie2 is free software: you can redistribute it and/or 
- *	modify it under the terms of the GNU General Public License as published 
+ *	devilspie2 is free software: you can redistribute it and/or
+ *	modify it under the terms of the GNU General Public License as published
  *	by the Free Software Foundation, either version 3 of the License, or
  *	(at your option) any later version.
- * 
+ *
  *	devilspie2 is distributed in the hope that it will be useful,
  *	but WITHOUT ANY WARRANTY; without even the implied warranty of
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
- * 
+ *
  *	You should have received a copy of the GNU General Public License
- *	along with devilspie2.  
+ *	along with devilspie2.
  *	If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -23,6 +23,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 #include <glib/gi18n.h>
 
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
@@ -36,9 +37,6 @@
 #include "script.h"
 #include "script_functions.h"
 
-#if (GTK_MAJOR_VERSION >= 3)
-#define HAVE_GTK3
-#endif
 
 /**
  *
@@ -50,7 +48,7 @@ static gboolean emulate=FALSE;
 static gboolean show_version=FALSE;
 
 #ifdef HAVE_GTK3
-	// libwnck Version Information is only availible if you have 
+	// libwnck Version Information is only availible if you have
 	// libwnck 3.0 or later
 	static gboolean show_wnck_version=FALSE;
 #endif
@@ -65,15 +63,15 @@ static GSList *file_list=NULL;
  *
  */
 static void window_opened_cb(WnckScreen *screen, WnckWindow *window) {
-	
+
 	GSList *temp_list=file_list;
 	// set the window to work on
 	set_current_window(window);
-	
+
 	// for every file in the folder:	// print the list
-	
+
 	if (file_list!=NULL) {
-		
+
 		while(temp_list) {
 
 			// is it a LUA file?
@@ -105,7 +103,7 @@ void init_screens()
 	const int num_screens=gdk_display_get_n_screens(gdk_display_get_default());
 	for (i=0;i<num_screens;i++) {
 		WnckScreen *screen=wnck_screen_get(i);
-		
+
 		g_signal_connect(screen,"window_opened",(GCallback)window_opened_cb,NULL);
 	}
 }
@@ -117,7 +115,7 @@ void init_screens()
 void devilspie_exit()
 {
 	done_script();
-	
+
 	if (temp_folder!=NULL) g_free(temp_folder);
 }
 
@@ -130,7 +128,7 @@ static void signal_handler(int sig)
 	printf("\n");
 	printf(_("Received signal:"));
 	printf(" %d (%s)\n", sig, strsignal(sig));
-	
+
 	done_script_error_messages();
 
 	if (sig==SIGINT)
@@ -139,6 +137,18 @@ static void signal_handler(int sig)
 }
 
 
+/**
+ * filename_list_sortfunc
+ *   function to sort the inserted filenames, to be able to determine
+ *   which order files are loaded.
+ */
+gint filename_list_sortfunc(gconstpointer a,gconstpointer b)
+{
+	return g_ascii_strcasecmp(a,b);
+}
+
+
+
 /**
  *
  */
@@ -146,52 +156,56 @@ void load_scripts()
 {
 	GDir *dir;
 	const gchar *current_file;
-	
+
 	// add all the files in the script_folder to the file_list
 	dir=g_dir_open(script_folder,0,NULL);
 	if (!g_file_test(script_folder,G_FILE_TEST_IS_DIR)) {
 
 		printf(_("script_folder isn't a folder."));
 		printf("\n");
-		
+
 		devilspie_exit();
 		exit(EXIT_FAILURE);
 	}
-	
+
 	// a temp list so we dont ruin the start of the list that is stored
 	// in file_list
 	GSList *temp_list=file_list;
-	
+
 	int number_of_files=0;
-	
+
 	// add the files in the folder to our linked list
 	while ((current_file=g_dir_read_name(dir))) {
 
 		// we only bother with *.lua in the folder
 		if (g_str_has_suffix(current_file,".lua")) {
-			temp_list=g_slist_prepend(temp_list,g_build_path(G_DIR_SEPARATOR_S,g_get_user_config_dir(),"devilspie2",current_file,NULL));
+			temp_list=g_slist_insert_sorted(temp_list,
+			                          g_build_path(G_DIR_SEPARATOR_S,
+			                                       g_get_user_config_dir(),
+			                                       "devilspie2",
+			                                       current_file,NULL),filename_list_sortfunc);
 			number_of_files++;
 		}
 	}
-	
+
 	file_list=temp_list;
-	
+
 	g_dir_close(dir);
-	
+
 	if (number_of_files==0) {
 		printf(_("No script files found in the script folder - exiting."));
 		printf("\n\n");
 		exit(EXIT_SUCCESS);
 	}
-	
+
 	// print the list of files:
-	
+
 	if (debug) {
 		printf(_("List of LUA files in folder:"));
 		printf("\n");
-		
+
 		if (file_list!=NULL) {
-		
+
 			while(temp_list) {
 
 				// is it a LUA file?
@@ -211,39 +225,39 @@ void load_scripts()
 int main(int argc, char *argv[])
 {
 	static const GOptionEntry options[]={
-		{ "debug",			'd',	0,	G_OPTION_ARG_NONE,		&debug,			
+		{ "debug",			'd',	0,	G_OPTION_ARG_NONE,		&debug,
 			N_("Print debug info to stdout")},
-		{ "emulate",		'e',	0,	G_OPTION_ARG_NONE,		&emulate,		
+		{ "emulate",		'e',	0,	G_OPTION_ARG_NONE,		&emulate,
 			N_("Don't apply any rules, only emulate execution")},
-		{ "folder",			'f',	0,	G_OPTION_ARG_STRING,		&script_folder, 
+		{ "folder",			'f',	0,	G_OPTION_ARG_STRING,		&script_folder,
 			N_("Search for scripts in this folder"),N_("FOLDER")},
 		{ "version",		'v',	0,	G_OPTION_ARG_NONE,		&show_version,
 			N_("Show Devilspie2 version and quit")},
 #ifdef HAVE_GTK3
-		// libwnck Version Information is only availible if you have 
+		// libwnck Version Information is only availible if you have
 		// libwnck 3.0 or later
 		{ "wnck-version",	'w',	0,	G_OPTION_ARG_NONE,		&show_wnck_version,
 			N_("Show libwnck version and quit")},
 #endif
 		{ NULL }
 	};
-	
+
 	GError *error=NULL;
 	GOptionContext *context;
-	
+
 	// Init gettext stuff
 	setlocale(LC_ALL,"");
-	
+
 	bindtextdomain(PACKAGE,LOCALEDIR);
 	bind_textdomain_codeset(PACKAGE,"");
 	textdomain(PACKAGE);
-	
+
 	gdk_init(&argc, &argv);
-	
+
 	gchar *devilspie2_description=NULL;
-	
+
 	devilspie2_description=g_strdup_printf(_("apply rules on windows"));
-	
+
 	gchar *full_desc_string=g_strdup_printf("- %s",devilspie2_description);
 
 	context=g_option_context_new(full_desc_string);
@@ -253,15 +267,15 @@ int main(int argc, char *argv[])
 		printf("\n");
 		exit(EXIT_FAILURE);
 	}
-	
+
 	g_free(full_desc_string);
 	g_free(devilspie2_description);
 
 	// if the folder is NULL, default to ~/.config/devilspie2/
 	if (script_folder==NULL) {
-		
+
 		temp_folder=g_build_path(G_DIR_SEPARATOR_S,g_get_user_config_dir(),"devilspie2",NULL);
-		
+
 		// check if the folder does exist
 		if (!g_file_test(temp_folder,G_FILE_TEST_IS_DIR)) {
 
@@ -272,53 +286,65 @@ int main(int argc, char *argv[])
 				exit(EXIT_FAILURE);
 			}
 		}
-		
+
 		script_folder=temp_folder;
 	}
-	
+
 	if (show_version) {
 		printf("Devilspie2 v%s\n\n",DEVILSPIE2_VERSION);
 		exit(EXIT_SUCCESS);
 	}
 #ifdef HAVE_GTK3
-	// libwnck Version Information is only availible if you have 
+	// libwnck Version Information is only availible if you have
 	// libwnck 3.0 or later
 	if (show_wnck_version) {
-		printf("libwnck v%d.%d.%d\n\n",WNCK_MAJOR_VERSION,WNCK_MINOR_VERSION,WNCK_MICRO_VERSION);
+		printf("libwnck v%d.%d.%d\n\n",
+		       WNCK_MAJOR_VERSION,
+		       WNCK_MINOR_VERSION,
+		       WNCK_MICRO_VERSION);
 		exit(EXIT_SUCCESS);
 	}
 #endif
-	
+
+#if (GTK_MAJOR_VERSION >= 3)
+	if (!GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
+		printf(_("An X11 display is required for devilspie2."));
+		printf("\n\n");
+		return EXIT_FAILURE;
+	}
+
+#endif
+
 	if (debug) {
-		
+
 		if (emulate) {
 			printf(_("Running devilspie2 in debug and emulate mode."));
 		} else {
 			printf(_("Running devilspie2 in debug mode."));
 		}
-		
+
 		printf("\n\n");
-		
+
 		printf(_("Using scripts from folder: %s"),script_folder);
-		
+
 		printf("\n");
-		
+
 		devilspie2_debug=TRUE;
 	}
-	
+
 	// Should we only run an emulation (don't modify any windows)
 	if (emulate) devilspie2_emulate=emulate;
-	
+
 	if (init_script_error_messages()!=0) {
 		printf(_("Couldn't init script error messages!"));
 		printf("\n");
 		exit(EXIT_FAILURE);
 	}
-	
+
 	load_scripts();
-	
+
 	if (debug) printf("------------\n");
-	
+
 	// remove stuff cleanly
 	atexit(devilspie_exit);
 
@@ -333,7 +359,7 @@ int main(int argc, char *argv[])
 	}
 
 	init_screens();
-	
+
 	loop=g_main_loop_new(NULL,TRUE);
 	g_main_loop_run(loop);
 
